/*----- serial.hpp*/

/**

        serial port handler class

        definitions

**/

#ifndef SERIAL_HPP

#define SERIAL_HPP

extern "C" {
        int  FOSSILinit(int p);
        void FOSSILclose(int p);
        int  FOSSILset(int p, unsigned int s);
        void FOSSILdtr(int p, int s);
        int  FOSSILget(int p);
        int  FOSSILput(int p, int c);
        int  FOSSILstatus(int p);
        int  FOSSILnext(int p);
        void FOSSILpout(int p);
        void FOSSILpin(int p);
        void FOSSILfout(int p);
        void FOSSILflow(int s);
        void FOSSILbreak(int p, int s);
        void FOSSILxmit(int p, int s);
        void FOSSILwatch(int p, int s);
        int  FOSSILwrite(int p, void *b, int n);
        int  FOSSILread(int p, void *b, int n);
};

enum Parity { None, Check, Odd, Even };
enum Flow { No = 0, TXon = 1, CTS = 2, RXon = 8, Read = 4 };
enum Switch { Off, On, What = -1 };

class Serial {

private:

        union {
                unsigned int i;
                struct {
                        unsigned int wl:2;
                        unsigned int sb:1;
                        unsigned int ps:2;
                        unsigned int sp:3;
                } b;
        } ps;

        int port;               // port this instance is tied to
        int spd;                // baud rate / 100
        int wdln;               // wordlength
        int stpbts;             // stop bits
        Parity prty;            // parity
        Switch dtrval;          // current setting of dtr
        int fossil;             // highest func supported by fossil
        Switch xmit;            // transmitter on or off?
        Flow flow;              // flow control settings
        Switch watch;           // is watchdog on?

public:

        /* transmitter */

        int tbc(void) { return ((FOSSILstatus(port) & 0x2000) != 0); };
        void put(int c) { FOSSILput(port,c); };
        void write(void *b, int n);             // write n bytes from b to
xmit
buf
        void purgeOut(void) { FOSSILpout(port); };
        void flushOut(void) { FOSSILfout(port); };
        Switch xmitter(Switch s = What);// turn on/off transmitter

        /* receiver */

        int rbc(void) { return ((FOSSILstatus(port) & 0x100) != 0); };
        int next(void) { return FOSSILnext(port); };
        int get(void) { return FOSSILget(port); };
        void read(void *b,int n);               // read up to n bytes into b
        void purgeIn(void) { FOSSILpin(port); };
        Switch cd(void) { return (Switch) ((FOSSILstatus(port) & 0x80) != 0);
};
        int error(void) { return ((FOSSILstatus(port) & 0x200) != 0); };

        /* control */

        Switch dtr(Switch s = What);    // get or set dtr
        void    brk(int l = 100);               // break for l msecs
        Switch watchdog(Switch s = What);// turn on/off watchdog
        Flow flowCtrl(Flow s = Read);   // get/set flow control
        int speed(int s = 0);           // get or set baud rate
        int word(int w = 0);            // get or set word length
        int stopbits(int b = 0);        // get or set stopbits
        Parity parity(Parity p = Check);// get or set parity

        /* initialization/cleanup */


        /* constructor / destructor */

        Serial(int n = 0,
               int b = 24,                      // initialize serial port n
to speed
               int w = 8,                       // b, wordlength w, stopbits
s and
               int s = 2,                       // parity p
               Parity p = None);

        ~Serial(void) { FOSSILclose(port); };

};

/*----- serial.cpp */

/**

        serial port handler class

        implementation

**/

#include <serial.hpp>
#include <time.h>       // msleep prototype

static int brates[8] = { 192,384,3,6,12,24,48,96 };

Serial::Serial(int n,int b,int w,int s,Parity p)

{
        int i;

        port = n;
        spd = b;
        wdln = w;
        stpbts = s;
        prty = p;

        if (FOSSILinit(n))
                dtrval = On;
        else
                dtrval = Off;

        xmit = On;
        flow = No;
        ps.b.wl = w - 5;
        ps.b.sb = (s == 2);
        ps.b.ps = (unsigned int) p;

        for (i=0; i < 8; i++)
                if (brates[i] == b)
                        break;

        ps.b.sp = (i==8)?5:i;
        FOSSILset(n,ps.i);
};

Switch Serial::dtr(Switch s)

{
        if (s != What) {
                FOSSILdtr(port,(int) s);
                dtrval = s;
        }
        return dtrval;
};

Flow Serial::flowCtrl(Flow s)

{
        if (s != Read) {
                flow = (Flow) ((unsigned int) flow ^ (unsigned int) s);
                FOSSILflow((int) flow);
        };

        return flow;
};

void Serial::brk(int l)

{
        if (fossil < 0x1a)
                return;

        FOSSILbreak(port,1);
        msleep(l);
        FOSSILbreak(port,0);
};

Switch Serial::xmitter(Switch s)

{
        if (s != What) {
                xmit = s;
                FOSSILxmit(port,(int)s);
        };

        return xmit;
}

Switch Serial::watchdog(Switch s)

{
        if (s != What) {
                watch = s;
                FOSSILwatch(port,(int) s);
        };

        return watch;
};

Parity Serial::parity(Parity p)

{
        if (p != Check) {
                ps.b.ps = (unsigned int) p;
                prty = p;
                FOSSILset(port,ps.i);
        }
        return prty;
};

int Serial::speed(int s)

{
        int i;

        if (s) {
                for (i = 0; (i < 8) && (brates[i] != s); i++) ;
                ps.b.sp = (i==8)?5:i;
                FOSSILset(port,ps.i);
        };

        return spd;
};

int Serial::word(int w)

{

        if (w) {
                ps.b.wl = w;
                wdln = w;
                FOSSILset(port,ps.i);
        };

        return wdln;
};

int Serial::stopbits(int b)

{
        if (b) {
                ps.b.sb = b;
                stpbts = b;
                FOSSILset(port,ps.i);
        };

        return stpbts;
};

void Serial::write(void *b, int n)

{
        if (fossil < 0x19) {
                int i = 0;

                while (i++ < n)
                        put((int) *(char *)b++);
        }
        else
                FOSSILwrite(port,b,n);
}

void Serial::read(void *b, int n)

{
        if (fossil < 0x18) {
                int i = 0;

                while ((i++ < n) && next())
                        *(char *)b++ = get();
        }
        else
                FOSSILread(port,b,n);
}



/*----- fossil.asm */

INCLUDE MACROS.ASM


begcode FOSSIL

;int FOSSILinit(int p);

c_public FOSSILinit

func FOSSILinit

        push bp
        mov bp,sp

        mov dx,p[bp]
        mov ax,0400h
        int 14h

        cmp ax,1954h
        jne ifail

        mov al,bl
        xor ah,ah

        pop bp
        ret

ifail:  mov ax,0
        pop bp
        ret

c_endp FOSSILinit

;void FOSSILclose(int p);

c_public FOSSILclose

func FOSSILclose

        push bp
        mov bp,sp
        mov ax,0500h
        mov dx,p[bp]
        int 14h

        pop bp
        ret

c_endp FOSSILclose

;int  FOSSILset(int p, unsigned int s);

c_public FOSSILset

func FOSSILset

        push bp
        mov bp,sp

        mov dx,p[bp]
        mov ax,p+2[bp]
        int 14h

        pop bp
        ret

c_endp FOSSILset

;void FOSSILdtr(int p, int s);

c_public FOSSILdtr

func FOSSILdtr

        push bp
        mov bp,sp

        mov dx,p[bp]
        mov ax,p+2[bp]
        mov ah,6h
        int 14h

        pop bp
        ret

c_endp  FOSSILdtr

;int FOSSILstatus(int p);

c_public FOSSILstatus

func FOSSILstatus

        push bp
        mov bp,sp

        mov dx,p[bp]
        mov ax,0300h
        int 14h

        pop bp
        ret

c_endp FOSSILstatus

;int  FOSSILget(int p);

c_public FOSSILget

func FOSSILget

        push bp
        mov bp,sp

        mov dx,p[bp]
        mov ax,0200h
        int 14h

        pop bp
        ret

c_endp FOSSILget

;int  FOSSILput(int p, int c);

c_public FOSSILput

func FOSSILput

        push bp
        mov bp,sp

        mov dx,p[bp]
        mov ax,p+2[bp]
        mov ah,1
        int 14h

        pop bp
        ret

c_endp FOSSILput

;int FOSSILnext(int p);

c_public FOSSILnext

func FOSSILnext

        push bp
        mov bp,sp

        mov dx,p[bp]
        mov ax,0c00h
        int 14h

        not ah
        and al,ah
        xor ah,ah

        pop bp
        ret

c_endp FOSSILnext

;void FOSSILpout(int p);

c_public FOSSILpout

func FOSSILpout

        push bp
        mov bp,sp

        mov ax,0900h
        mov dx,p[bp]
        int 14h

        pop bp
        ret

c_endp FOSSILpout

;void FOSSILpin(int p);

c_public FOSSILpin

func FOSSILpin

        push bp
        mov bp,sp

        mov dx,p[bp]
        mov ax,0a00h
        int 14h

        pop bp
        ret

c_endp FOSSILpin

;void FOSSILfout(int p);

c_public FOSSILfout

func FOSSILfout

        push bp
        mov bp,sp

        mov ax,0800h
        mov dx,p[bp]
        int 14h

        pop bp
        ret

c_endp FOSSILfout

;void FOSSILflow(int s);

c_public FOSSILflow

func FOSSILflow

        push bp
        mov bp,sp

        mov ax,p+2[bp]
        mov dx,p[bp]
        mov ah,0fh
        int 14h

        pop bp
        ret

c_endp FOSSILflow

;void FOSSILbreak(int p, int s);

c_public FOSSILbreak

func FOSSILbreak

        push bp
        mov bp,sp

        mov dx,p[bp]
        mov ax,p+2[bp]
        mov ah,1ah
        int 14h

        pop bp
        ret

c_endp  FOSSILbreak

;void FOSSILxmit(int p, int s);

c_public FOSSILxmit

func FOSSILxmit

        push bp
        mov bp,sp

        mov dx,p[bp]
        mov ax,p+2[bp]
        shl ax,1
        mov ah,10h
        int 14h

        pop bp
        ret

c_endp FOSSILxmit

;void FOSSILwatch(int p, int s)

c_public FOSSILwatch

func FOSSILwatch

        push bp
        mov bp,sp

        mov dx, p[bp]
        mov ax, p+2[bp]
        mov ah,14h
        int 14h

        pop bp
        ret

c_endp FOSSILwatch

;int FOSSILwrite(int p, void *b, int n);

c_public FOSSILwrite

func FOSSILwrite

        push bp
        mov bp,sp
        push di

        mov dx,p[bp]

IF SPTR
        push ds
        pop es
        mov di,p+2[bp]
        mov cx,p+4[bp]
ELSE
        les di,p+2[bp]
        mov cx,p+6[bp]
ENDIF
        mov ax,1900h
        int 14h

        pop di
        pop bp
        ret

c_endp FOSSILwrite

;int FOSSILread(int p, void *b, int n);

c_public FOSSILread

func FOSSILread

        push bp
        mov bp,sp
        push di

        mov dx,p[bp]

IF SPTR
        push ds
        pop es
        mov di,p+2[bp]
        mov cx,p+4[bp]
ELSE
        les di,p+2[bp]
        mov cx,p+6[bp]
ENDIF
        mov ax,1800h
        int 14h

        pop di
        pop bp
        ret

c_endp FOSSILread

endcode FOSSIL
end

/*----- terminal.cpp */

#include <bios.h>
#include <time.h>
#include <serial.hpp>
#include <window.hpp>

int main(int argc, char **argv)

{
        unsigned int c;
        int running = 1;
        window_set ws1(0);
        Serial port(0,24);

        window status(&ws1,0,0,10,40,0x1C,0);
        window text(&ws1,11,0,disp_numrows-1,disp_numcols-1,0x17,1);

        status.clear();
        text.clear();

        status.title("Status");
        text.title("Terminal");
        text.locate(0,0);

        status.locate(0,0);
        status.printf("%s (%d00,%d,",port.dtr()?"port initialized":"error
initializing port",port.speed(),port.word());
        status.putch((port.parity()==None)?'n':(port.parity()==Odd)?'o':'e');
        status.printf(",%d)",port.stopbits());

        while (running) {

                int cd = -1;
                int errors = -1;

                while (!bioskey(1)) {
                        if (port.cd() != cd) {
                                status.locate(1,0);
                                status.puts(port.cd()?"carrier detect":"no
carrier    ");
                                cd = port.cd();
                        }

                        if (port.error() != errors) {
                                status.locate(2,0);
                                status.puts(port.error()?"line error":"

 ");
                                errors = port.error();
                        }

                        if (port.next())
                                text.putch(port.get());
                }

                switch (c = bioskey(0)) {
                        case 0x4400 :           // <F10> hangs up
                                port.dtr(Off);

                        case 0x2d00 :           // <ALT><X> stays on line
                                status.locate(3,0);
                                status.puts("later dude...");
                                running = 0;
                                break;

                        case 0x2e00 :
                                port.brk();
                                break;

                        case 0x2300 :
                                status.locate(3,0);
                                status.puts("hang up");
                                port.dtr(Off);
                                msleep(500);
                                port.dtr(On);
                                status.locate(3,0);
                                status.puts("       ");
                                break;

                        default:
                                if (c & 0xff)
                                        port.put(c & 0xff);
                }
        }

        return 0;
}
